#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import pipeline
from yagocd.resources import Base
import mock
import pytest


class TestPipelineEntity(object):
    def test_has_all_managers_methods(self):
        excludes = ['tie_pipelines', 'list', 'find']

        def get_public_methods(klass):
            methods = set()
            for name in dir(klass):
                if name.startswith('_'):
                    continue

                candidate = getattr(klass, name)
                if hasattr(candidate, '__call__'):
                    methods.add(name)
            return methods

        managers_methods = get_public_methods(pipeline.PipelineManager)
        entity_methods = get_public_methods(pipeline.PipelineEntity)
        result = managers_methods - entity_methods - set(excludes)
        assert len(result) == 0, "Some methods are missing in pipeline entity: {}".format(result)

    @pytest.fixture()
    def pipeline_entity(self, mock_session):
        return pipeline.PipelineEntity(
            session=mock_session,
            data={'name': 'pipeline_1',
                  'materials': [
                      {'description': 'child_name_1', 'type': 'Pipeline'},
                      {'description': 'git_repo_1', 'type': 'Git'},
                      {'description': 'child_name_2', 'type': 'Pipeline'},
                  ]
                  },
            group='baz'
        )

    def test_entity_is_not_none(self, pipeline_entity):
        assert pipeline_entity is not None

    def test_is_instance_of_base(self, pipeline_entity):
        assert isinstance(pipeline_entity, Base)

    def test_reading_group(self, pipeline_entity):
        assert pipeline_entity.group == 'baz'

    def test_predecessors_empty(self, pipeline_entity):
        assert pipeline_entity.predecessors == list()

    def test_set_predecessors(self, pipeline_entity):
        value = [1, 3, 5]
        pipeline_entity.predecessors = value
        assert pipeline_entity.predecessors == value
        assert pipeline_entity.predecessors != pipeline_entity.descendants

    def test_get_predecessors(self, mock_session, pipeline_entity):
        child_1 = pipeline.PipelineEntity(mock_session, data=dict(name='child_1'))
        child_2 = pipeline.PipelineEntity(mock_session, data=dict(name='child_2'))
        child_3 = pipeline.PipelineEntity(mock_session, data=dict(name='child_3'))
        child_4 = pipeline.PipelineEntity(mock_session, data=dict(name='child_4'))

        child_2.predecessors = [child_1]
        child_3.predecessors = [child_1, child_2]
        child_4.predecessors = [child_3]
        pipeline_entity.predecessors = [child_4]

        assert pipeline_entity.get_predecessors() == [child_4]
        assert (sorted(pipeline_entity.get_predecessors(True), key=lambda x: x.data.name) ==
                [child_1, child_2, child_3, child_4])

        assert child_4.predecessors == [child_3]
        assert (sorted(child_4.get_predecessors(True), key=lambda x: x.data.name) ==
                [child_1, child_2, child_3])

        assert child_3.predecessors == [child_1, child_2]
        assert (sorted(child_3.get_predecessors(True), key=lambda x: x.data.name) ==
                [child_1, child_2])

        assert child_2.predecessors == [child_1]
        assert (sorted(child_2.get_predecessors(True), key=lambda x: x.data.name) == [child_1])

        assert child_1.predecessors == []
        assert (sorted(child_1.get_predecessors(True), key=lambda x: x.data.name) == [])

    def test_descendants_empty(self, pipeline_entity):
        assert pipeline_entity.descendants == list()

    def test_set_descendants(self, pipeline_entity):
        value = [2, 4, 8]
        pipeline_entity.descendants = value
        assert pipeline_entity.descendants == value
        assert pipeline_entity.predecessors != pipeline_entity.descendants

    def test_get_descendants(self, mock_session, pipeline_entity):
        parent_1 = pipeline.PipelineEntity(mock_session, data=dict(name='parent_1'))
        parent_2 = pipeline.PipelineEntity(mock_session, data=dict(name='parent_2'))
        parent_3 = pipeline.PipelineEntity(mock_session, data=dict(name='parent_3'))
        parent_4 = pipeline.PipelineEntity(mock_session, data=dict(name='parent_4'))

        parent_3.descendants = [parent_4]
        parent_2.descendants = [parent_4, parent_3]
        parent_1.descendants = [parent_2]
        pipeline_entity.descendants = [parent_1]

        assert pipeline_entity.get_descendants() == [parent_1]
        assert (sorted(pipeline_entity.get_descendants(True), key=lambda x: x.data.name) ==
                [parent_1, parent_2, parent_3, parent_4])

        assert parent_1.descendants == [parent_2]
        assert (sorted(parent_1.get_descendants(True), key=lambda x: x.data.name) ==
                [parent_2, parent_3, parent_4])

        assert parent_2.descendants == [parent_4, parent_3]
        assert (sorted(parent_2.get_descendants(True), key=lambda x: x.data.name) ==
                [parent_3, parent_4])

        assert parent_3.descendants == [parent_4]
        assert (sorted(parent_3.get_descendants(True), key=lambda x: x.data.name) == [parent_4])

        assert parent_4.descendants == []
        assert (sorted(parent_4.get_descendants(True), key=lambda x: x.data.name) == [])

    def test_get_url(self, pipeline_entity):
        assert (
            pipeline_entity.get_url('http://example.com', 'test_name') ==
            'http://example.com/go/tab/pipeline/history/test_name'
        )

    def test_url(self, pipeline_entity):
        assert pipeline_entity.url == 'http://example.com/go/tab/pipeline/history/pipeline_1'

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_history_call(self, history_mock, pipeline_entity):
        pipeline_entity.history()
        history_mock.assert_called_with(name=pipeline_entity.data.name, offset=0)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_full_history_call(self, history_mock, pipeline_entity):
        history_mock.side_effect = [['foo', 'bar', 'baz'], []]
        list(pipeline_entity.full_history())
        calls = [mock.call(pipeline_entity.data.name, 0), mock.call(pipeline_entity.data.name, 3)]
        history_mock.assert_has_calls(calls)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_last_call(self, history_mock, pipeline_entity):
        pipeline_entity.last()
        history_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_last_returns_last(self, history_mock, pipeline_entity):
        history_mock.return_value = ['50', '30', '10']
        assert pipeline_entity.last() == '50'

    @mock.patch('yagocd.resources.pipeline.PipelineManager.get')
    def test_get_call(self, get_mock, pipeline_entity):
        pipeline_entity.get()
        get_mock.assert_called_with(name=pipeline_entity.data.name, counter=0)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.status')
    def test_status_call(self, status_mock, pipeline_entity):
        pipeline_entity.status()
        status_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.pause')
    def test_pause_call(self, pause_mock, pipeline_entity):
        pipeline_entity.pause('custom-reason')
        pause_mock.assert_called_with(name=pipeline_entity.data.name, cause='custom-reason')

    @mock.patch('yagocd.resources.pipeline.PipelineManager.unpause')
    def test_unpause_call(self, unpause_mock, pipeline_entity):
        pipeline_entity.unpause()
        unpause_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.release_lock')
    def test_release_lock_call(self, release_lock_mock, pipeline_entity):
        pipeline_entity.release_lock()
        release_lock_mock.assert_called_with(name=pipeline_entity.data.name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.schedule')
    def test_schedule_call(self, schedule_mock, pipeline_entity):
        pipeline_entity.schedule()
        schedule_mock.assert_called_with(name=pipeline_entity.data.name, materials=None, variables=None,
                                         secure_variables=None)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.schedule_with_instance')
    def test_schedule_with_instance_call(self, schedule_with_instance_mock, pipeline_entity):
        pipeline_entity.schedule_with_instance()
        schedule_with_instance_mock.assert_called_with(name=pipeline_entity.data.name, materials=None, variables=None,
                                                       secure_variables=None, backoff=0.5, max_tries=20)


class TestGraphDepthWalk(object):
    @pytest.mark.parametrize("root, expected", [
        ('a', ['a', 'c']),
        ('b', ['a', 'c', 'b']),
        ('c', ['c']),
        ('d', ['a', 'c', 'd']),
        ('e', ['a', 'c', 'b', 'e']),
    ])
    def test_graph(self, root, expected):
        graph = {'a': ['c'], 'b': ['a'], 'c': [], 'd': ['a'], 'e': ['b', 'a']}

        assert len(pipeline.PipelineEntity.graph_depth_walk(root, lambda x: graph.get(x))) == len(expected)
        assert sorted(pipeline.PipelineEntity.graph_depth_walk(root, lambda x: graph.get(x))) == sorted(expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import time

from yagocd.resources import BaseManager, Base


class ArtifactManager(BaseManager):
    """
    The artifacts API allows users to query and create artifacts of a job.
    """

    def __init__(
        self,
        session,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Constructs instance of ``ArtifactManager``.
        Parameters to the constructor and methods of the class could be duplicated. That is because of two use cases
        of this class:
            1. When the class being instantiated from :class:`yagocd.client.Client`, we don't know all the necessary
             parameters yet, but we need an instance to work with. So we skip parameters instantiation in constructor,
             but require them for each method.
            2. When the class being used from :class:`yagocd.resources.job.JobInstance` - in this case we already
            know all required parameters, so we can instantiate `ArtifactManager` with them.

        :param session: session object from client.
        :type session: yagocd.session.Session.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        """
        super(ArtifactManager, self).__init__(session)

        self.base_api = self._session.base_api(api_path='')

        self._pipeline_name = pipeline_name
        self._pipeline_counter = pipeline_counter
        self._stage_name = stage_name
        self._stage_counter = stage_counter
        self._job_name = job_name

    def list(
        self,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None
    ):
        """
        Lists all available artifacts in a job.

        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: An array of :class:`yagocd.resources.artifact.Artifact`.
        :rtype: list of yagocd.resources.artifact.Artifact
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.get(
            path=(
                '{base_api}'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}.json'
            ).format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name
            ),
        )
        artifacts = list()
        for data in response.json():
            artifacts.append(Artifact(session=self._session, data=data))

        return artifacts

    def directory(
        self,
        path,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None,
    ):
        """
        Gets an artifact directory by its path.

        :note: The `path` can be a nested directory for e.g. target/dist.zip

        :warning: Since it may take an undetermined amount of time to compress
        a directory, the server may return a `202 Accepted` code to indicate that
        it is compressing the requested directory.
        Users are expected to poll the url every few seconds to check if the
        directory is available.

        :param path: path to directory.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return:
            * A status code `202 Accepted` to acknowledge your request to compress
              the contents of the requested directory.
            * The requested directory contents in the form of a zip file.
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.get(
            path=(
                '{base_api}'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{path_to_folder}').format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name,
                path_to_folder=path
            ),
        )

        return response.content

    def directory_wait(
        self,
        path,
        timeout=60,
        backoff=0.4,
        max_wait=4,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None,
    ):
        """
        Gets an artifact directory by its path.
        This method wraps original `directory` method, adding
        timeout to wait  for directory to be available.

        :param path: path to directory.
        :param timeout: timeout in seconds to wait for directory.
        :param backoff: backoff value.
        :param max_wait: maximum wait amount.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: The requested directory contents in the form of a zip file.
        """
        start_time = time.time()
        time_elapsed = 0
        counter = 0
        directory_zip = None

        while time_elapsed < timeout:
            directory_zip = self.directory(path, pipeline_name, pipeline_counter, stage_name, stage_counter, job_name)
            if directory_zip:
                break

            time.sleep(min(backoff * (2 ** counter), max_wait))
            counter += 1
            time_elapsed = time.time() - start_time

        return directory_zip

    def create(
        self,
        path,
        filename,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None,
    ):
        """
        Uploads a local file as an artifact.

        :param path: path to the file within job directory.
        :param filename: the contents file to be uploaded.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: an acknowledgement that the file was created.
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.post(
            path=(
                '{base_api}'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{path_to_file}'
            ).format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name,
                path_to_file=path
            ),
            files={'file': open(filename, 'rb')}
        )

        return response.text

    def append(
        self,
        path,
        filename,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
        job_name=None,
    ):
        """
        Appends a local file to an existing artifact.

        :param path: path to the file within job directory.
        :param filename: the contents file to be uploaded.
        :param pipeline_name: name of the pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of the stage.
        :param stage_counter: stage counter.
        :param job_name: name of the job.
        :return: an acknowledgement that the file was created.
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter
        assert self._job_name or job_name

        response = self._session.put(
            path=(
                '{base_api}'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{path_to_file}'
            ).format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter,
                job_name=self._job_name or job_name,
                path_to_file=path
            ),
            files={'file': open(filename, 'rb')}
        )

        return response.text


class Artifact(Base):
    def files(self):
        return [ArtifactFile(session=self._session, data=data) for data in self.data.files]


class ArtifactFile(Base):
    def fetch(self):
        response = self._session.get(self.data.url)
        return response.content



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import os
import sys
import time
import copy
import shutil
import subprocess

import mock
import pytest
import requests
from vcr import VCR
from yagocd import Yagocd
from yagocd.session import Session


@pytest.fixture(scope='session')
def tests_dir():
    return os.path.dirname(os.path.realpath(__file__))


@pytest.fixture()
def mock_session():
    session = mock.patch('yagocd.session.Session').start()
    session.server_url = 'http://example.com'
    return session


@pytest.fixture()
def session_fixture():
    options = copy.deepcopy(Yagocd.DEFAULT_OPTIONS)
    options['server'] = 'http://local.docker:8153/'
    return Session(
        auth=('admin', '12345'),
        options=options
    )


my_vcr_object = VCR(
    path_transformer=VCR.ensure_suffix('.yaml'),
    cassette_library_dir=os.path.join(tests_dir(), 'fixtures/cassettes'),
)


@pytest.fixture()
def my_vcr():
    return my_vcr_object


CONTAINER_NAME = 'yagocd-server'
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))


def pytest_addoption(parser):
    parser.addoption(
        "--fresh-run", action="store_true", default=False,
        help=(
            "Execute fresh run of tests: all cassettes will be deleted, instance of GoCD will be "
            "started in Docker container and test would be executed upon it."
        )
    )


@pytest.fixture(scope='session')
def fresh_run(request):
    return request.config.getoption("--fresh-run")


@pytest.fixture(scope="session", autouse=True)
def gocd_docker(request, fresh_run):
    if fresh_run:
        if os.path.exists(my_vcr_object.cassette_library_dir):
            print("Removing existing cassettes...")
            shutil.rmtree(my_vcr_object.cassette_library_dir)

        start_container()
        wait_till_started()

        def fin():
            stop_container()

        request.addfinalizer(fin)


def start_container():
    sys.stdout.write('Starting Docker container...')
    output = subprocess.check_output([
        "/usr/local/bin/docker",
        "ps",
        "--quiet",
        "--filter=name={container_name}".format(
            container_name=CONTAINER_NAME),
    ])

    if not output:
        subprocess.check_call([
            "/usr/local/bin/docker",
            "run",
            "--detach",
            "--net=host",
            "--volume={current_dir}/docker:/workspace".format(
                current_dir=CURRENT_DIR
            ),
            "--workdir=/workspace",
            "--name={container_name}".format(
                container_name=CONTAINER_NAME
            ),
            "gocd/gocd-dev",
            "/bin/bash",
            "-c",
            "'/workspace/bootstrap.sh'",
        ])


def wait_till_started():
    sys.stdout.write('Waiting for availability of GoCD in Docker.')
    while True:
        try:
            requests.get("http://local.docker:8153/go")
        except requests.exceptions.ConnectionError:
            sys.stdout.write('.')
            time.sleep(5)
        else:
            break


def stop_container():
    subprocess.check_call([
        "/usr/local/bin/docker",
        "rm",
        "-f",
        CONTAINER_NAME
    ])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import pytest
from six import string_types

from yagocd.resources import user


class BaseTestUserManager(object):
    @pytest.fixture()
    def manager(self, session_fixture):
        return user.UserManager(session=session_fixture)


class TestList(BaseTestUserManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("user/list") as cass:
            manager.list()
            assert cass.requests[0].path == '/go/api/users'

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("user/list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("user/list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("user/list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("user/list"):
            result = manager.list()
            assert all(isinstance(u, user.UserEntity) for u in result)


class TestGet(BaseTestUserManager):
    USERNAME = 'admin'

    def test_get_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("user/get") as cass:
            manager.get(self.USERNAME)
            assert cass.requests[0].path == '/go/api/users/{0}'.format(self.USERNAME)

    def test_get_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("user/get") as cass:
            manager.get(self.USERNAME)
            assert cass.requests[0].method == 'GET'

    def test_get_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("user/get") as cass:
            manager.get(self.USERNAME)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_get_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("user/get") as cass:
            manager.get(self.USERNAME)
            assert cass.responses[0]['status']['code'] == 200

    def test_get_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("user/get"):
            result = manager.get(self.USERNAME)
            assert isinstance(result, user.UserEntity)


#
#
# class TestCreate(BaseTestUserManager):
#     OPTIONS = {
#         'login_name': 'foobar',
#         'enabled': True,
#         'email': 'foo@bar.baz',
#         'email_me': False,
#         'checkin_aliases': ['foo', 'bar', 'baz']
#     }
#
#     def test_create_request_url(self, manager, my_vcr):
#         with my_vcr.use_cassette("user/create") as cass:
#             manager.create(self.OPTIONS)
#             assert cass.requests[0].path == '/go/api/users'
#
#     def test_create_request_method(self, manager, my_vcr):
#         with my_vcr.use_cassette("user/create") as cass:
#             manager.create(self.OPTIONS)
#             assert cass.requests[0].method == 'POST'
#
#     def test_create_request_accept_headers(self, manager, my_vcr):
#         with my_vcr.use_cassette("user/create") as cass:
#             manager.create(self.OPTIONS)
#             assert cass.requests[0].headers['accept'] == 'application/json'
#
#     def test_create_response_code(self, manager, my_vcr):
#         with my_vcr.use_cassette("user/create") as cass:
#             manager.create(self.OPTIONS)
#             assert cass.responses[0]['status']['code'] == 200
#
#     def test_create_return_type(self, manager, my_vcr):
#         with my_vcr.use_cassette("user/create"):
#             result = manager.create(self.OPTIONS)
#             assert isinstance(result, user.UserEntity)


class TestUpdate(BaseTestUserManager):
    USERNAME = 'admin'
    OPTIONS = {
        'email': 'foo@bar.baz'
    }

    def test_update_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update") as cass:
            manager.update(self.USERNAME, self.OPTIONS)
            assert cass.requests[0].path == '/go/api/users/{0}'.format(self.USERNAME)

    def test_update_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update") as cass:
            manager.update(self.USERNAME, self.OPTIONS)
            assert cass.requests[0].method == 'PATCH'

    def test_update_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update") as cass:
            manager.update(self.USERNAME, self.OPTIONS)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_update_request_content_type_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update") as cass:
            manager.update(self.USERNAME, self.OPTIONS)
            assert cass.requests[0].headers['Content-Type'] == 'application/json'

    def test_update_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update") as cass:
            manager.update(self.USERNAME, self.OPTIONS)
            assert cass.responses[0]['status']['code'] == 200

    def test_update_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update"):
            result = manager.update(self.USERNAME, self.OPTIONS)
            assert isinstance(result, user.UserEntity)

    def test_update_return_value(self, manager, my_vcr):
        with my_vcr.use_cassette("user/update"):
            result = manager.update(self.USERNAME, self.OPTIONS)
            assert result.data.email == self.OPTIONS['email']


class TestDelete(BaseTestUserManager):
    USERNAME = 'user_02'

    def test_delete_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("user/delete") as cass:
            manager.delete(self.USERNAME)
            assert cass.requests[0].path == '/go/api/users/{0}'.format(self.USERNAME)

    def test_delete_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("user/delete") as cass:
            manager.delete(self.USERNAME)
            assert cass.requests[0].method == 'DELETE'

    def test_delete_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("user/delete") as cass:
            manager.delete(self.USERNAME)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_delete_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("user/delete") as cass:
            manager.delete(self.USERNAME)
            assert cass.responses[0]['status']['code'] == 200

    def test_delete_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("user/delete"):
            result = manager.delete(self.USERNAME)
            assert isinstance(result, string_types)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import BaseManager, Base


class MaterialManager(BaseManager):
    """
    The materials API allows users to query and notify materials in the Go configuration.
    """

    def list(self):
        """
        Lists all available materials, these are materials that are present in the in ``cruise-config.xml``.
        :return: An array of :class:`yagocd.resources.material.MaterialEntity`.
        :rtype: list of yagocd.resources.material.MaterialEntity
        """
        response = self._session.get(
            path='{base_api}/config/materials'.format(base_api=self.base_api),
            headers={'Accept': 'application/json'},
        )

        materials = list()
        for data in response.json():
            materials.append(MaterialEntity(session=self._session, data=data))

        return materials

    def modifications(self, fingerprint, offset=0):
        """
        Get modifications of specific material.

        :param fingerprint: fingerprint of material.
        :param offset: number of modifications to be skipped.
        :return: A list of modification objects :class:`yagocd.resources.material.ModificationEntity`.
        :rtype: list of yagocd.resources.material.ModificationEntity
        """
        response = self._session.get(
            path='{base_api}/materials/{fingerprint}/modifications/{offset}'.format(
                base_api=self.base_api,
                fingerprint=fingerprint,
                offset=offset
            ),
            headers={'Accept': 'application/json'},
        )

        modifications = list()
        for data in response.json().get('modifications', {}):
            modifications.append(ModificationEntity(session=self._session, data=data))

        return modifications

    def notify_svn(self, uuid):
        """
        APIs that notify Go Server when a commit has been made in Version Control
        and Go needs to trigger relevant pipelines.

        [!!!] When using this feature, uncheck Poll for new changes or
        set autoUpdate flag in cruise configuration to false for the relevant material.
        Otherwise you will get:
           requests.exceptions.HTTPError: 404 Client Error:
           Not Found for url: http://localhost:8153/go/api/material/notify/svn

        :param uuid: The subversion repository UUID.
        :return: A text confirmation.
        """
        response = self._session.post(
            path='{base_api}/material/notify/svn'.format(base_api=self.base_api),
            data={'uuid': uuid},
            headers={
                'Accept': 'application/json',
                'Confirm': 'true'
            },
        )

        return response.text

    def notify_git(self, url):
        """
        APIs that notify Go Server when a commit has been made in Version Control
        and Go needs to trigger relevant pipelines.

        [!!!] When using this feature, uncheck Poll for new changes or
        set autoUpdate flag in cruise configuration to false for the relevant material.
        Otherwise you will get:
           requests.exceptions.HTTPError: 404 Client Error:
           Not Found for url: http://localhost:8153/go/api/material/notify/git

        :param url: The git repository url as defined in cruise-config.xml.
        :return: A text confirmation.
        """
        response = self._session.post(
            path='{base_api}/material/notify/git'.format(base_api=self.base_api),
            data={'repository_url': url},
            headers={
                'Accept': 'application/json',
                'Confirm': 'true'
            },
        )

        return response.text

    def notify_hg(self, url):
        """
        APIs that notify Go Server when a commit has been made in Version Control
        and Go needs to trigger relevant pipelines.

        [!!!] When using this feature, uncheck Poll for new changes or
        set autoUpdate flag in cruise configuration to false for the relevant material.
        Otherwise you will get:
           requests.exceptions.HTTPError: 404 Client Error:
           Not Found for url: http://localhost:8153/go/api/material/notify/hg

        :param url: The git repository url as defined in cruise-config.xml.
        :return: A text confirmation.
        """
        response = self._session.post(
            path='{base_api}/material/notify/hg'.format(base_api=self.base_api),
            data={'repository_url': url},
            headers={
                'Accept': 'application/json',
                'Confirm': 'true'
            },
        )

        return response.text


class MaterialEntity(Base):
    pass


class ModificationEntity(Base):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import zipfile
from six import BytesIO
from six import string_types, binary_type

from yagocd.resources import artifact

import pytest
import mock


class BaseTestArtifactManager(object):
    PIPELINE_NAME = 'Shared_Services'
    PIPELINE_COUNTER = 7
    STAGE_NAME = 'Commit'
    STAGE_COUNTER = '1'
    JOB_NAME = 'build'

    @pytest.fixture()
    def manager(self, session_fixture):
        return artifact.ArtifactManager(
            session=session_fixture,
            pipeline_name=self.PIPELINE_NAME,
            pipeline_counter=self.PIPELINE_COUNTER,
            stage_name=self.STAGE_NAME,
            stage_counter=self.STAGE_COUNTER,
            job_name=self.JOB_NAME
        )


class TestList(BaseTestArtifactManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list") as cass:
            manager.list()
            assert cass.requests[0].path == (
                '/go'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}.json'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME
            )

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list"):
            result = manager.list()
            assert isinstance(result, list)

    def test_list_returns_instances_of_artifact(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_list"):
            result = manager.list()
            assert all(isinstance(i, artifact.Artifact) for i in result)


class TestDirectory(BaseTestArtifactManager):
    DIRECTORY_PATH = 'path/to/the/.zip'

    def test_directory_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_not_ready") as cass:
            manager.directory(path=self.DIRECTORY_PATH)
            assert cass.requests[0].path == (
                '/go'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{dir_path}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME,
                dir_path=self.DIRECTORY_PATH
            )

    def test_directory_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_not_ready") as cass:
            manager.directory(path=self.DIRECTORY_PATH)
            assert cass.requests[0].method == 'GET'

    def test_directory_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_not_ready") as cass:
            manager.directory(path=self.DIRECTORY_PATH)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_directory_response_code_not_ready(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_not_ready") as cass:
            manager.directory(path=self.DIRECTORY_PATH)
            assert cass.responses[0]['status']['code'] == 202

    def test_directory_return_type_not_ready(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_not_ready"):
            result = manager.directory(path=self.DIRECTORY_PATH)
            assert isinstance(result, (string_types, binary_type))

    def test_directory_response_code_ready(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_ready") as cass:
            manager.directory(path=self.DIRECTORY_PATH)
            assert cass.responses[0]['status']['code'] == 200

    def test_directory_return_type_ready(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_ready"):
            result = manager.directory(path=self.DIRECTORY_PATH)
            assert isinstance(result, (string_types, binary_type))

    def test_directory_return_is_zip_file(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_ready"):
            result = manager.directory(path=self.DIRECTORY_PATH)

            myzipfile = zipfile.ZipFile(BytesIO(result))
            assert myzipfile.testzip() is None


class TestDirectoryWait(BaseTestArtifactManager):
    DIRECTORY_PATH = 'path/to/.zip'

    @mock.patch('yagocd.resources.artifact.ArtifactManager.directory')
    def test_tie_descendants_is_called(self, mock_directory, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/artifact_directory_wait"):
            manager.directory_wait(path=self.DIRECTORY_PATH)
            mock_directory.assert_called()

    def test_directory_wait_response_code_first(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_wait") as cass:
            manager.directory_wait(path=self.DIRECTORY_PATH)
            assert cass.responses[0]['status']['code'] == 202

    def test_directory_wait_response_code_last(self, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_directory_wait") as cass:
            manager.directory_wait(path=self.DIRECTORY_PATH)
            assert cass.responses[-1]['status']['code'] == 200


class TestCreate(BaseTestArtifactManager):
    PATH_TO_FILE = 'path/to/the/file.txt'
    FILE_CONTENT = 'Sample test data.\nFoo and Bar.'

    @pytest.fixture(scope='session')
    def sample_artifact(self, tmpdir_factory):
        fn = tmpdir_factory.mktemp("gocd").join("artifact.txt")
        fn.write(self.FILE_CONTENT)
        return fn

    def test_create_request_url(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create") as cass:
            manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].path == (
                '/go'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{path_to_file}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME,
                path_to_file=self.PATH_TO_FILE
            )

    def test_create_request_method(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create") as cass:
            manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].method == 'POST'

    def test_create_request_accept_headers(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create") as cass:
            manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_create_response_code(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create") as cass:
            manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.responses[0]['status']['code'] == 201

    def test_create_return_type(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create"):
            result = manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert isinstance(result, string_types)

    def test_create_return_value(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_create"):
            result = manager.create(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert result == 'File {0} was created successfully'.format(self.PATH_TO_FILE)


class TestAppend(BaseTestArtifactManager):
    PATH_TO_FILE = 'path/to/the/file-append.txt'
    FILE_CONTENT = 'Data to append.'

    @pytest.fixture(scope='session')
    def sample_artifact(self, tmpdir_factory):
        fn = tmpdir_factory.mktemp("gocd").join("artifact-append.txt")
        fn.write(self.FILE_CONTENT)
        return fn

    def test_append_request_url(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append") as cass:
            manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].path == (
                '/go'
                '/files'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{path_to_file}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME,
                path_to_file=self.PATH_TO_FILE
            )

    def test_append_request_method(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append") as cass:
            manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].method == 'PUT'

    def test_append_request_accept_headers(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append") as cass:
            manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_append_response_code(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append") as cass:
            manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert cass.responses[0]['status']['code'] == 200

    def test_append_return_type(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append"):
            result = manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert isinstance(result, string_types)

    def test_append_return_value(self, sample_artifact, manager, my_vcr):
        with my_vcr.use_cassette("artifact/artifact_append"):
            result = manager.append(path=self.PATH_TO_FILE, filename=sample_artifact.strpath)
            assert result == 'File {0} was appended successfully'.format(self.PATH_TO_FILE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import re

# noinspection PyUnresolvedReferences
from six.moves import html_parser

from yagocd.resources import BaseManager


class AboutPageTableParser(html_parser.HTMLParser):
    def __init__(self):
        html_parser.HTMLParser.__init__(self)
        self._in_td = False
        self.data = list()

    def handle_starttag(self, tag, attrs):
        if tag == 'td':
            self._in_td = True

    def handle_data(self, data):
        if self._in_td:
            self.data.append(data)

    def handle_endtag(self, tag):
        self._in_td = False


class InfoManager(BaseManager):
    """
    Class for getting general information about GoCD server.

    Right now this class just parses /about page, as there is no REST API for this purpose.
    After it will be implemented, this class should be rewritten.

    TODO: rewrite after https://github.com/gocd/gocd/issues/2202 would be fixed.
    """

    SERVER_VERSION_RE = re.compile(r'Server Version')
    VERSION_NUMBER_RE = re.compile(r'[\d.-]+')

    JVM_VERSION_RE = re.compile(r'JVM version')
    OS_INFO_RE = re.compile(r'OS Information')
    ARTIFACT_FREE_SPACE_RE = re.compile(r'Usable space in artifacts repository')
    DB_SCHEMA_VERSION_RE = re.compile(r'Database schema version')

    def __init__(self, session):
        super(InfoManager, self).__init__(session)
        self._parsed = None

    def _get_about_page(self):
        result = self._session.get(
            path='go/about',
            headers={'Accept': 'text/html'}
        )
        return result.text

    def _get_value(self, regex):
        if not self._parsed:
            content = self._get_about_page()

            parser = AboutPageTableParser()
            parser.feed(content)

            self._parsed = dict(zip(parser.data[0::2], parser.data[1::2]))

        for title, value in self._parsed.items():
            if regex.search(title):
                return value

    @property
    def version(self):
        value = self._get_value(self.SERVER_VERSION_RE)
        if value:
            match = self.VERSION_NUMBER_RE.match(value)
            if match:
                return match.group()

    @property
    def jvm_version(self):
        return self._get_value(self.JVM_VERSION_RE)

    @property
    def os_info(self):
        return self._get_value(self.OS_INFO_RE)

    @property
    def artifact_free_space(self):
        return self._get_value(self.ARTIFACT_FREE_SPACE_RE)

    @property
    def db_schema_version(self):
        return self._get_value(self.DB_SCHEMA_VERSION_RE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import pytest

from yagocd import Yagocd
from yagocd.resources import agent, artifact, configuration, feed, job, info, material, pipeline, property as prop, \
    stage, user


class TestInit(object):
    def test_default(self):
        try:
            Yagocd()
        except Exception as e:
            assert False, "Unexpected exception raised: {0}".format(e)

    def test_set_server(self):
        go = Yagocd(server='http://example.com')
        assert go.server_url == 'http://example.com'

    def test_set_server_from_options(self):
        go = Yagocd(options=dict(server='http://example.com'))
        assert go.server_url == 'http://example.com'

    def test_set_verify(self):
        go = Yagocd(options=dict(verify=False))
        assert go._session._options['verify'] == False

    def test_set_headers(self):
        go = Yagocd(options=dict(headers=dict(Accept='foo/bar')))
        assert go._session._options['headers']['Accept'] == 'foo/bar'


class TestServerUrl(object):
    def test_default(self):
        assert Yagocd().server_url == 'http://localhost:8153'


class TestManagers(object):
    @pytest.fixture()
    def go_fixture(self):
        return Yagocd()

    def test_agents(self, go_fixture):
        assert isinstance(go_fixture.agents, agent.AgentManager)

    def test_artifacts(self, go_fixture):
        assert isinstance(go_fixture.artifacts, artifact.ArtifactManager)

    def test_configurations(self, go_fixture):
        assert isinstance(go_fixture.configurations, configuration.ConfigurationManager)

    def test_feeds(self, go_fixture):
        assert isinstance(go_fixture.feeds, feed.FeedManager)

    def test_jobs(self, go_fixture):
        assert isinstance(go_fixture.jobs, job.JobManager)

    def test_info(self, go_fixture):
        assert isinstance(go_fixture.info, info.InfoManager)

    def test_materials(self, go_fixture):
        assert isinstance(go_fixture.materials, material.MaterialManager)

    def test_pipelines(self, go_fixture):
        assert isinstance(go_fixture.pipelines, pipeline.PipelineManager)

    def test_properties(self, go_fixture):
        assert isinstance(go_fixture.properties, prop.PropertyManager)

    def test_stages(self, go_fixture):
        assert isinstance(go_fixture.stages, stage.StageManager)

    def test_users(self, go_fixture):
        assert isinstance(go_fixture.users, user.UserManager)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import info

import mock
import pytest


class BaseTestInfoManager(object):
    @pytest.fixture()
    def manager(self, session_fixture):
        return info.InfoManager(session=session_fixture)


class TestVersion(BaseTestInfoManager):
    def test_result_is_not_none(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page"):
            result = manager.version
            assert result is not None

    def test_multiple_call(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page") as cass:
            for x in range(10):
                result = manager.version
                assert result is not None

            assert len(cass.requests) == 1

    @mock.patch('yagocd.resources.info.InfoManager._get_value')
    def test_unmatched_version(self, get_value_mock, manager, my_vcr):
        get_value_mock.return_value = 'unsupported-version-number'

        with my_vcr.use_cassette("info/about_page"):
            result = manager.version
            assert result is None


class TestJvmVersion(BaseTestInfoManager):
    def test_result_is_not_none(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page"):
            result = manager.jvm_version
            assert result is not None

    def test_multiple_call(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page") as cass:
            for x in range(10):
                result = manager.jvm_version
                assert result is not None

            assert len(cass.requests) == 1


class TestOsInfo(BaseTestInfoManager):
    def test_result_is_not_none(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page"):
            result = manager.os_info
            assert result is not None

    def test_multiple_call(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page") as cass:
            for x in range(10):
                result = manager.os_info
                assert result is not None

            assert len(cass.requests) == 1


class TestArtifactFreeSpace(BaseTestInfoManager):
    def test_result_is_not_none(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page"):
            result = manager.artifact_free_space
            assert result is not None

    def test_multiple_call(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page") as cass:
            for x in range(10):
                result = manager.artifact_free_space
                assert result is not None

            assert len(cass.requests) == 1


class TestDbSchemaVersion(BaseTestInfoManager):
    def test_result_is_not_none(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page"):
            result = manager.db_schema_version
            assert result is not None

    def test_multiple_call(self, manager, my_vcr):
        with my_vcr.use_cassette("info/about_page") as cass:
            for x in range(10):
                result = manager.db_schema_version
                assert result is not None

            assert len(cass.requests) == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from easydict import EasyDict


class BaseManager(object):
    def __init__(self, session):
        """
        :type session: yagocd.session.Session
        """
        self._session = session
        self.base_api = self._session.base_api()


class Base(object):
    def __init__(self, session, data):
        self._session = session
        self._data = EasyDict(data)

        self.base_api = self._session.base_api()

    @property
    def data(self):
        return self._data

    def __str__(self):
        return self.data.__str__()

    def __repr__(self):
        return self.data.__repr__()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import pytest
from six import string_types

from yagocd.resources import configuration


class BaseTestConfigurationManager(object):

    @pytest.fixture()
    def manager(self, session_fixture):
        return configuration.ConfigurationManager(session=session_fixture)


class TestModifications(BaseTestConfigurationManager):
    def test_modifications_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/modifications") as cass:
            manager.modifications()
            assert cass.requests[0].path == '/go/api/config/revisions'

    def test_modifications_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/modifications") as cass:
            manager.modifications()
            assert cass.requests[0].method == 'GET'

    def test_modifications_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/modifications") as cass:
            manager.modifications()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_modifications_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/modifications") as cass:
            manager.modifications()
            assert cass.responses[0]['status']['code'] == 200


class TestDiff(BaseTestConfigurationManager):
    start = '3336409d194252e8c82799472196e70cbe4c7916'
    end = '719be0bb99470b79fc48a949c24845f65a0fd8ef'

    def test_diff_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/diff") as cass:
            manager.diff(self.start, self.end)
            assert cass.requests[0].path == '/go/api/config/diff/{0}/{1}'.format(
                self.start, self.end
            )

    def test_diff_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/diff") as cass:
            manager.diff(self.start, self.end)
            assert cass.requests[0].method == 'GET'

    def test_diff_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/diff") as cass:
            manager.diff(self.start, self.end)
            assert cass.requests[0].headers['accept'] == 'text/plain'

    def test_diff_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/diff") as cass:
            manager.diff(self.start, self.end)
            assert cass.responses[0]['status']['code'] == 200

    def test_diff_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/diff"):
            result = manager.diff(self.start, self.end)
            assert isinstance(result, string_types)


class TestConfigCurrent(BaseTestConfigurationManager):
    def test_config_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_current") as cass:
            manager.config()
            assert cass.requests[0].path == '/go/api/admin/config/current.xml'

    def test_config_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_current") as cass:
            manager.config()
            assert cass.requests[0].method == 'GET'

    def test_config_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_current") as cass:
            manager.config()
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_config_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_current") as cass:
            manager.config()
            assert cass.responses[0]['status']['code'] == 200

    def test_config_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_current"):
            result = manager.config()
            assert isinstance(result, string_types)


class TestConfigMD5(BaseTestConfigurationManager):
    md5 = '412f48f7e2ff254e47564a6852ed7a2e'

    def test_config_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_md5") as cass:
            manager.config(self.md5)
            assert cass.requests[0].path == '/go/api/admin/config/{0}.xml'.format(self.md5)

    def test_config_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_md5") as cass:
            manager.config(self.md5)
            assert cass.requests[0].method == 'GET'

    def test_config_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_md5") as cass:
            manager.config(self.md5)
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_config_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_md5") as cass:
            manager.config(self.md5)
            assert cass.responses[0]['status']['code'] == 200

    def test_config_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("configuration/config_md5"):
            result = manager.config(self.md5)
            assert isinstance(result, string_types)
